
import box

import ika
from data import data 
import timer

images = {'U': ika.Image('images\\tap-up.png'),
			'D': ika.Image('images\\tap-down.png'),
			'L': ika.Image('images\\tap-left.png'),
			'R': ika.Image('images\\tap-right.png')
			}

class Tap(box.Rectangle):
	def __init__(self, x, y, dir, tile, scene):
		self.dir = dir
		self.on = False
		self.liquid = Liquid(tile)
		self.leakSpeed = 2
		self.adjustment = 0
		self.scene = scene
		self.timer = timer.Timer(75)
		
		self.tile = tile
		self.myTiles = tile.myTiles
		#for some reason 'is' doesn't work?????? (but this does)
		if dir in 'U':
			box.Rectangle.__init__(self, x*data.tileSize, y*data.tileSize, data.tileSize, data.tileSize*2)
		elif dir in 'D':
			box.Rectangle.__init__(self, x*data.tileSize, y*data.tileSize - data.tileSize, data.tileSize, data.tileSize*2)
		elif dir in 'R':
			box.Rectangle.__init__(self, x*data.tileSize - data.tileSize, y*data.tileSize, data.tileSize*2, data.tileSize)
		elif dir in 'L':
			box.Rectangle.__init__(self, x*data.tileSize, y*data.tileSize, data.tileSize*2, data.tileSize)
		
		
	def Update(self):
		if not self.on or not self.timer.IsDone():
			return
		
		self.timer.Reset()
		
		if self.scene.liquidAllowed >= self.leakSpeed:
			self.liquid.Add(self.leakSpeed + self.adjustment)
			self.scene.liquidAllowed -= self.leakSpeed
		
		elif self.scene.liquidAllowed < self.leakSpeed:
			self.liquid.Add(self.scene.liquidAllowed + self.adjustment)
			self.scene.liquidAllowed = 0
		
		self.liquid.Update()
				
		self.adjustment += 1
		
	def Render(self):
		global images
		
		if self.on:
			ika.Video.TintBlit(images[self.dir], self.x, self.y, ika.RGB(0,255,0,255))
		else:
			ika.Video.Blit(images[self.dir], self.x, self.y)
		#ika.Video.DrawRect(self.x, self.y, self.x +  self.w, self.y+self.h, data.colors['WaterBlueHalf'], 1)
		
		
	def ReceivedClick(self):
		##if able to be turned on...
		if self.on:
			self.on = False
			self.scene.numTapsOn -=1
		else:
			if self.scene.numTapsOn < 2:
				self.on = True
				self.scene.numTapsOn += 1
		
	

		
class Gate(box.Rectangle):
	def __init__(self, x, y, tile, scene):
		tile.HasGate()
		self.x = x*data.tileSize
		self.y = y*data.tileSize
		self.scene = scene
		self.tile = tile
		
		box.Rectangle.__init__(self, self.x, self.y, data.tileSize, data.tileSize)
		
	def ReceivedClick(self):
		if self.tile.wallFlag:
			self.tile.ToggleGate()
			self.scene.numGatesUp -= 1
		else:
			if self.scene.numGatesUp < 2:
				self.tile.ToggleGate()
				self.scene.numGatesUp += 1

class Finish(object):
	def __init__(self, tile, scene):
		self.x = tile.x
		self.y = tile.y
		self.tile = tile
		self.scene = scene
	
	def Okay(self):
		if self.tile.currentWater >= self.tile.maxWater:
			return True
		return False
		
class Liquid(object):
	def __init__(self, tile, amount = 0):
		self.amount = amount
		self.dead = False
		self.dropTile = tile
		##self.tilesFull = []
		##self.tilesVisited = []
		##self.currentTile = 0
		##self.tilesVisited.append(tile)
	
	def Update(self):
		assert self.amount >= 0, 'liquid amount less than zero'
		if self.amount == 0:
			return
		
		self.dropTile.AddLiquid(self.amount)
		self.amount = 0
		'''
		print 'Updating Liquid.'
		print 'currentTile', self.currentTile
		#find more tiles
		for tile in self.tilesVisited:	##
			print tile
			if not tile.checked:
				if tile.u:
					if not FindTile(tile.u.x, tile.u.y, self.tilesFull):
						MyUniqueAddAndSort(self.tilesVisited, tile.u)
				if tile.l:
					if not FindTile(tile.l.x, tile.l.y, self.tilesFull):
						MyUniqueAddAndSort(self.tilesVisited, tile.l)
				if tile.r:
					if not FindTile(tile.r.x, tile.r.y, self.tilesFull):
						MyUniqueAddAndSort(self.tilesVisited, tile.r)
				if tile.d:
					if not FindTile(tile.d.x, tile.d.y, self.tilesFull):
						MyUniqueAddAndSort(self.tilesVisited, tile.d)
				tile.checked = True
				
		#add liquid to tiles
		flaggedList = []
		for i, tile in enumerate(self.tilesVisited):
			if self.amount >= 2:
				amt, flagged = tile.AddLiquid(2)	#returns: remaining amount, waterflag
			else:
				amt, flagged = tile.AddLiquid(self.amount)	#returns: remaining amount, waterflag
			
			self.amount += amt
			
			#maintain a list of which tiles to get rid of
			if flagged:	
				flaggedList.append(i)
				
			if self.amount <= 0:
				break
				
				
		#use list to add to full list while removing from current list - reversed
		for i in reversed(flaggedList):
			self.tilesFull.append(self.tilesVisited.pop(i))
		'''	
		
	def Add(self, amount):
		self.amount += amount
		
		
def MyUniqueAddAndSort(array, obj):
	if len(array) is 0:
		array.append(obj)
		return
	
	if array[0].x > obj.x:
		array.insert(0,obj)
		
	i = 0
	#from the front
	while i < len(array):
		if array[i].x == obj.x:
			break
		if array[i].x > obj.x:
			i -= 1
			break
		i += 1
		
	while i < len(array):
		if array[i].x > obj.x:
			break
		if array[i].y == obj.y and array[i].x == obj.x:
			return
		if array[i].y > obj.y:
			break
		i += 1
		
	
	print 'INSERTING', obj.x, obj.y
	
	array.insert(i, obj)
	
def FindTile(x, y, tiles):
	i = 0
	xFound = False
	while i < len(tiles):
		if x is tiles[i].x:
			xFound = True
			break
		if x < tiles[i].x:
			break
		i += 1
	
	if not xFound:
		return None
	
	while i < len(tiles):
		if x != tiles[i].x:
			break
		if y is tiles[i].y:
			assert x is tiles[i].x, "code thinks same tile--not true"
			return tiles[i]
		if y < tiles[i].y:
			break
		i += 1
		
	return None
			