import ika
from data import data

tiles = []

class Map(object):
	def __init__(self, currentLevel):
		self.mapName = self.ConvertLevelToMapName(currentLevel)
		ika.Map.Switch(self.mapName)
		
		global tiles
		tiles = []
		
		self.MakeTiles()

	
	def Update(self):
		pass		
		
	def Render(self):
		global tiles
		for tile in tiles:
			tile.Render()
	
	
	def ConvertLevelToMapName(self, level):
		return 'maps\\' + str(level) + '.ika-map'
	
	def MakeTiles(self):
		height = int(ika.Map.height/data.tileSize)	#tilesize
		width = int(ika.Map.width/data.tileSize)
		
		#find first tile that is a real tile (not background)
		#and from it, all the others will be born
		stop = False
		for i in range(height):
			for j in range(width):
				##print 'tile', j,i, ika.Map.GetTile(j,i, 0)
				if ika.Map.GetTile(j,i, 0) is not 0:##change this!!
					Tile(j, i, 'x')
					stop = True
					break
			if stop:
				break
	
	

class Tile(object):
	def __init__(self, x, y, origin , parentTile = None, max = 2):
		self.maxWater = max
		self.currentWater = 0
		self.wallFlag = False	#true if a wall or when gate triggered
		self.waterFlag = False	#triggered when full of water
		self.gateFlag = False
		self.x = x	#in number of tiles
		self.y = y
		self.realX = x*data.tileSize
		self.realY = y*data.tileSize
		
		self.u = None
		self.d = None
		self.r = None
		self.l = None
		self.myTiles = []		
		
		global tiles
		MyAddAndSort(tiles, self)
		
		if origin is 'u':
			self.u = parentTile
			self.u.d = self
			self.r = self.MakeRightTile(x, y, origin )	
			self.l = self.MakeLeftTile(x, y, origin )
			self.d = self.MakeDownTile(x, y, origin )		
		elif origin is 'd':
			self.d = parentTile
			self.d.u = self
			self.r = self.MakeRightTile(x, y, origin )
			self.l = self.MakeLeftTile(x, y, origin )
			self.u = self.MakeUpTile(x, y, origin )	
		elif origin is 'r':
			self.r = parentTile
			self.r.l = self
			self.d = self.MakeDownTile(x, y, origin )		
			self.u = self.MakeUpTile(x, y, origin )
			self.l = self.MakeLeftTile(x, y, origin )
		elif origin is 'l':
			self.l = parentTile
			self.l.r = self
			self.u = self.MakeUpTile(x, y, origin )
			self.d = self.MakeDownTile(x, y, origin )
			self.r = self.MakeRightTile(x, y, origin )
		else:
			self.u = self.MakeUpTile(x, y, origin )
			self.r = self.MakeRightTile(x, y, origin )
			self.d = self.MakeDownTile(x, y, origin )		
			self.l = self.MakeLeftTile(x, y, origin )
			self.d.u = None
			del self
			return
				
		if self.u:
			self.myTiles.append(self.u)
		if self.d:
			self.myTiles.append(self.d)
		if self.r:
			self.myTiles.append(self.r)
		if self.l:
			self.myTiles.append(self.l)
			
		
	def Update(self):
		tile = self.FindNeighborWithMostWater()
		
		if tile is None:
			return
		
		dif	= tile.currentWater - self.currentWater
		if dif <= 1:
			return
		if dif%2 is 0:
			div = int(dif/2)
		else:	
			div = int(dif/2) + 1		
			
		
		tile.GiveWaterTo(self, div)
		#tile.waterTakenFrom = true
	
	
	def FindNeighborWithMostWater(self):
		
		curMax = max(self.currentWater, 2)
		
		options = []
		
		for tile in self.myTiles:
			if tile.currentWater > curMax:
				curMax = tile.currentWater
			
		for tile in self.myTiles:
			if tile.currentWater == curMax:
				options.append(tile)
		
		if options == []:
			return None
		if len(options) == 1:
			return options[0]
		else:
			num = ika.Random(0, len(options))
			return options[num]
		'''
		return tile
		'''
		
	def Render(self):
		if self.gateFlag and self.wallFlag:
			ika.Video.DrawRect(self.realX, self.realY, self.realX + data.tileSize, self.realY + data.tileSize, data.colors['Orange'], 1)
		elif self.gateFlag and not self.wallFlag:
			ika.Video.DrawRect(self.realX, self.realY, self.realX + data.tileSize, self.realY + data.tileSize, data.colors['Orange'], 0)
	
		##real numbers--beware!!!
		elif self.currentWater == 1:
			ika.Video.DrawRect(self.realX, self.realY, self.realX + data.tileSize, self.realY + data.tileSize, data.colors['WaterBlueHalf'], 1)
		elif self.currentWater >= 2:
			ika.Video.DrawRect(self.realX, self.realY, self.realX + data.tileSize, self.realY + data.tileSize, data.colors['WaterBlueFull'], 1)
		
		#data.fonts['Interface'].Print(self.realX, self.realY, str(self.currentWater))
		
	def IsWall(self):
		self.wallFlag = True
		#rest shouldn't matter
		
	def HasGate(self):
		self.gateFlag = True
		self.maxWater = 0
		self.wallFlag = False
	
	def ToggleGate(self):
		if self.wallFlag is True:
			self.wallFlag = False
		else:
			self.wallFlag = True
			
	def GiveWaterTo(self, tile, amount):
		if tile.wallFlag:
			return
		
		tile.AddLiquid(amount)
		self.currentWater -= amount
		
		if self.currentWater < self.maxWater:
			self.waterFlag = False
		
	def AddLiquid(self, amount):
		self.currentWater += amount
		##add stuff for half water
		##if self.currentWater 
		if self.currentWater >= self.maxWater:
			self.waterFlag = True
			
		return self.waterFlag
			
	def MakeUpTile(self, x, y, origin ):
		if ika.Map.GetTile(x, y-1, 0) is 0:
			return None
		if ika.Map.GetObs(x,y-1, 0):
			return None
		
		tile = FindTile(x, y-1)
		if tile:
			return tile
		if self.l:
			if self.l.u:
				if self.l.u.r:
					return self.l.u.r
		if self.r:
			if self.r.u:
				if self.r.u.l:
					return self.r.u.l
		if self.u is None:	
			return Tile(x, y-1, 'd' , self)	#make a new tile
			
		
	def MakeLeftTile(self, x, y, origin ):
		if ika.Map.GetTile(x-1, y, 0) is  0:
			return None
		if ika.Map.GetObs(x-1,y, 0):
			return None
		tile = FindTile(x-1, y)
		if tile:
			return tile
		if self.u:
			if self.u.l:
				if self.u.l.d:
					return self.u.l.d
		if self.d:
			if self.d.l:
				if self.d.l.u:
					return self.d.l.u
		
		if self.l is None:
			return Tile(x-1, y, 'r' , self)
			
	def MakeRightTile(self, x, y, origin ):
		if ika.Map.GetTile(x+1, y, 0) is 0:
			return None
		if ika.Map.GetObs(x+1,y, 0):
			return None
		tile = FindTile(x+1, y)
		if tile:
			return tile
		if origin is not 'r':
			if self.u:
				if self.u.r:
					if self.u.r.d:
						return self.u.r.d
			if self.d:
				if self.d.r:
					if self.d.r.u:
						return self.d.r.u
			if self.r is None:	
				return Tile(x+1, y, 'l' , self)
			
	def MakeDownTile(self, x, y, origin ):
		if ika.Map.GetTile(x, y+1, 0) is 0:
			return None
		if ika.Map.GetObs(x,y+1, 0):
			return None
		tile = FindTile(x, y+1)
		if tile:
			return tile
		if origin is not 'd':
			if self.l:
				if self.l.d:
					if self.l.d.r:
						return self.l.d.r
		if self.r:
			if self.r.d:
				if self.r.d.l:
					return self.r.d.l
		if self.d is None:
			return Tile(x, y+1, 'u' , self)
		
	def __str__(self):
		exits = ''
		if self.u:
			exits += 'u'
		if self.d:
			exits += 'd'
		if self.r:
			exits += 'r'
		if self.l:
			exits += 'l'
		out = 'Tile: ' +  str(self.x) + ' ' +  str(self.y) + ' Exits:' + exits + ' CurWater ' + str(self.currentWater) + 'MaxWater' + str(self.maxWater)
		return out
		
def MyAddAndSort(array, obj):
	if len(array) is 0:
		array.append(obj)
		return
	
	if array[0].x > obj.x:
		array.insert(0,obj)
		
	i = 0
	#from the front
	while i < len(array):
		if array[i].x == obj.x:
			break
		if array[i].x > obj.x:
			#i -= 1
			break
		i += 1
		
	while i < len(array):
		if array[i].x > obj.x:
			break
		if array[i].y > obj.y:
			break
		i += 1
	
	array.insert(i, obj)

def FindTile(x, y):
	global tiles
	i = 0
	xFound = False
	while i < len(tiles):
		if x is tiles[i].x:
			xFound = True
			break
		if x < tiles[i].x:
			break
		i += 1
	
	if not xFound:
		return None
	
	while i < len(tiles):
		if x != tiles[i].x:
			break
		if y is tiles[i].y:
			assert x is tiles[i].x, "code thinks same tile--not true"
			return tiles[i]
		if y < tiles[i].y:
			break
		i += 1
		
	return None
			