import ika
from data import data
from game import game

"""
TODO:
-add option for menu to have a header (ie, if list['header']....)
-add option to display tooltips (probably best done by re-reversing dict 
	to normal and have it return key instead of value so that
	value would become tooltip... I was having fun and didn't see need for 
	the functionality -- an easy enough fix.)
-change position of menu to set x/y coords instead of
	trying to center on screen. That entire FigureAndSetXY() should probably
	be revamped.
	(user movable? ...no. that would be controlled by another class if/when immplemented) 
-tweak borders so a) not off center b) can be pretty
-add mouse functionality
-add functionality (possibly) such that a list (not dict) could be 
	used instead with blank tooltips (ie. probably just turn list into dict
	then procede as usual)
-possibly compact Select() into Menu() as indicated in note (near bottom)
"""



class Slot():
	def __init__(self, x, y, text, parent):
		self.x = x
		self.y = y
		
		self.margin = parent.margin
		self.font = parent.font
		self.cursor = parent.cursor
		
		self.text_x = self.x + parent.cursor_size + self.margin
		self.text_y = self.y + self.margin
		
		self.text = text
		self.selected = False
		
	def Render(self):
		if self.selected:
			self.cursor.Blit(self.x, self.text_y)
		self.font.Print(self.text_x, self.text_y, self.text)
	
	def __str__(self):
		out = "menu slot with text:" + self.text + '.end'
		return out
		
class Menu():
	def __init__(self, list = []):
		
		self.font = data.fonts["Interface"]
		
		global cursors
		self.cursor = data.cursors['default']	##ugly
		self.cursor_size = self.cursor.height
		
		#-Note: This dictionary is backwards.
		#-The keys are what is displayed, the program operates with the values.
		self.list = list
		self.header = ''
		
		self.menu_options = self.list ## #.keys()
		
		#none of these values are set in stone.. they are simply there to be initialized
		self.x = 0
		self.y = 0
		self.width = 0
		self.height = 0
		
		self.margin = 5 ##
		
		self.FigureAndSetXY()
		
		self.slots = []
		self.MakeSlots()
		
		self.current_selection = 0
		self.InitializeSelectedSlot()
		
		self.calls = None
		self.returns_to = None
		
		self.mask = ika.Image("images\\mask.png")
	
		for slot in self.slots:
			print slot
	def Update(self):
		if ika.Input.keyboard["UP"].Pressed():
			self.CursorUp()
		if ika.Input.keyboard["DOWN"].Pressed():
			self.CursorDown()
		if ika.Input.keyboard["RETURN"].Pressed():
			self.Select()
		if ika.Input.keyboard['ESCAPE'].Pressed():
			self.Return()
			
	def Render(self):
		ika.Video.Blit(self.mask, 0 ,0)
		ika.Video.DrawRect(self.x-1, self.y-1, self.x + self.width + 1, self.y + self.height + 1, data.colors['White'], 0)
		ika.Video.DrawRect(self.x, self.y, self.x + self.width, self.y + self.height, data.colors['Black'], 1)
		self.font.Print(100,100,self.header)
		
		for slot in self.slots:
			slot.Render()
		
	def FigureAndSetXY(self):
		font_height = self.font.height
		#figure x
		max_string_width = 0
		for option in self.menu_options:
			w = self.font.StringWidth(option)
			if w > max_string_width:
				max_string_width = w
		board_center_x = int(data.mapWidthPixels/2)
		#set x
		self.x = board_center_x - int(max_string_width/2) - self.margin - 8 ##see below me
		self.width = max_string_width + 3*self.margin + 16 ##16 = cursor size, should not be const
		
		#figure y
		needed_text_y = len(self.menu_options)*(font_height+1)
		board_center_y = int(data.mapHeightPixels/2)
		#set y
		self.y = board_center_y - int(needed_text_y/2) - self.margin
		self.height = needed_text_y + 2*self.margin
	
	def MakeSlots(self):
		x = self.x + self.margin
		y = self.y + self.margin
		font_height = self.font.height
		
		for option in self.menu_options:
			new_slot = Slot(x, y, option, self)
			self.slots.append(new_slot)
			y += font_height
	def InitializeSelectedSlot(self):
		self.slots[self.current_selection].selected = True
		
	def CursorUp(self):
		#note: does not act on a cursor, but rather tells selected slot to display cursor
		self.current_selection -= 1
		if self.current_selection < 0:
			self.current_selection += 1	#do nothing
		else:
			self.slots[self.current_selection].selected = True
			self.slots[self.current_selection+1].selected = False
		'''
		try:
			self.slots[self.current_selection].selected = True
			self.slots[self.current_selection+1].selected = False
		except:
			self.current_selection += 1
		'''
		
	def CursorDown(self):
		#note: does not act on a cursor, but rather tells selected slot to display cursor
		self.current_selection += 1
		if self.current_selection >= len(self.slots):
			self.current_selection -= 1
		else:
			self.slots[self.current_selection].selected = True
			self.slots[self.current_selection-1].selected = False
		
			
	
	def Select(self):
		key = self.list[self.current_selection]
		'''
		slot = self.slots[self.current_selection]
		key = slot.text
		#-program_code = self.list[key]
		#data.gains_control.remove(self)
		#self.calls(program_code)
		'''
		
		self.calls(key)
	
	def Return(self):
		#data.gains_control = [] #.remove(self)
		self.returns_to()
'''		
class SpeedMenu(Menu):
	def __init__(self):
		self.speeds = {'Slow': 24,
			'Medium': 18,
			'Fast': 12,
			'Very Fast': 6
			}
		Menu.__init__(self, self.speeds)
				
		self.returns_to = game.StartScreen
		
	def Select(self):
		
		data.gains_control.remove(self)
		slot = self.slots[self.current_selection]
		program_code = slot.text
		data.block_speed = self.speeds[program_code]
		
		from data import score
		score.speed = slot.text
'''
class PlayAsMenu(Menu):
	def __init__(self):
		list = ["Solids",
				"Liquids"
				]
				
		Menu.__init__(self, list)
	
		self.calls = game.PlayAsSelection
		#self.returns_to = game.SetScene('start menu')

from data import gameSettings
		
class StartMenu(Menu):
	def __init__(self):
		list = ["Play Game",
				"Play As",
				#"Load Game",
				"About"
				]
		
		Menu.__init__(self,list)
		
		self.calls = game.StartMenuSelection
		self.header = 'Currently playing as: \n\t\t ' + gameSettings.playingAs
		#self.returns_to = game.StartScreen
	def Render(self):
		self.header = 'Currently playing as: \n\t\t ' + gameSettings.playingAs
		ika.Video.Blit(self.mask, 0 ,0)
		ika.Video.DrawRect(self.x-1, self.y-1, self.x + self.width + 1, self.y + self.height + 1, data.colors['White'], 0)
		ika.Video.DrawRect(self.x, self.y, self.x + self.width, self.y + self.height, data.colors['Black'], 1)
		self.font.Print(100,100,self.header)
		
		for slot in self.slots:
			slot.Render()
		
class TryAgainMenu(Menu):
	def __init__(self):
		list = ["Try Again",
				"Main Menu",
				"Exit"
				]
		
		Menu.__init__(self,list)
		
		self.calls = game.TryAgainMenuSelection
		self.header = "You didn't succeed."
		
class PlayAgainMenu(Menu):
	def __init__(self):
		list = ["Continue Playing",
				"Main Menu",
				"Exit"
				]
		
		Menu.__init__(self,list)
		
		self.calls = game.PlayAgainMenuSelection
		self.header = "You succeeded."

class SaveProgressMenu(Menu):
	def __init__(self):
		list = ["Save Progress",
				"Continue"
				]
		
		Menu.__init__(self,list)
		
		self.calls = game.SaveProgressMenuSelection
