import ika
from mouse import mouse
from data import data

class Scene(object):
	def __init__(self):
		self.map = None
		self.takesOverList = []
		self.objects = []
		self.clickables = [] #should be of type Box
		
	def Update(self):
		
		length = len(self.takesOverList)
		if length > 0:
			self.takesOverList[length - 1].Update()
		else:
			for obj in self.objects:
				obj.Update()
				
	def Render(self):
		if self.map:
			ika.Map.Render()	#map
			self.map.Render()
		length = len(self.takesOverList)
		if length is not 0:
			self.takesOverList[length - 1].Render()
		else:
			for obj in self.objects:
				obj.Render()

import menu2
		
class StartScene(Scene):
	def __init__(self):
		Scene.__init__(self)
		
		menu = menu2.StartMenu()
		self.takesOverList.append(menu)
			
		"""
			Select Map
			Save
			Load
		"""

class WonScene(Scene):
	def __init__(self):
		Scene.__init__(self)
		
		menu = menu2.PlayAgainMenu()
		self.takesOverList.append(menu)
		
		##menu = menu2.SaveProgressMenu()
		##self.takesOverList.append(menu)
		
class LostScene(Scene):
	def __init__(self):
		Scene.__init__(self)
		
		menu = menu2.TryAgainMenu()
		self.takesOverList.append(menu)

import splash
		
class BeatGameScene(Scene):
	def __init__(self, player):
		Scene.__init__(self)
		
		
		
		text = "Contratulations! \n\n  You've beaten the game as " + player
		self.takesOverList.append(splash.Notice(text))
		
		
import map
import interactables
import shelf
import ai 

class GameScene(Scene):
	def __init__(self, player, mapName):
		Scene.__init__(self)
		self.objects.append(player)
		self.map = map.Map(mapName)
		
			
		self.taps = []
		self.gates = []
		self.finishes = []
		self.numTapsOn = 0
		self.numGatesUp = 0
		self.liquidAllowed = 0
		
		self.myShelf = shelf.Shelf('mapinfo.pydat')
		self.RetrieveMapInfo(mapName)
			
		#allow input for gate OR water
		if player.type is 'Solids':
			self.clickables += self.gates
		else:
			self.clickables += self.taps
		
		
		self.objects += self.taps
		
		
		self.max = len(map.tiles)
		self.first = int(self.max/2)
		#self.second = int(2*self.max/3)
		self.count = 0
		
		if player.type is "Liquids":
			self.objects.append(ai.SolidAI(self))
		else:
			self.objects.append(ai.LiquidAI(self))
		
		
	def Update(self):
		Scene.Update(self)
		
		if mouse.LeftClick():
			click = mouse.x, mouse.y
			for rect in self.clickables:
				if rect.ClickIn(click):
					rect.ReceivedClick()

		if self.count is 0:
			i = 0
			while i < self.max:
				map.tiles[i].Update()
				i += 2
			self.count = 1
		elif self.count is 1:
			i = 1
			while i < self.max:
				map.tiles[i].Update()
				i += 2
			self.count = 0
		
		okay = True
		for finish in self.finishes:
			if not finish.Okay():
				okay = False
				break
		if okay == True:
			from game import game
			game.LevelCompleted(okay)
		elif self.liquidAllowed <= 0:
			from game import game 
			game.LevelCompleted(okay)
			
	def	Render(self):
		Scene.Render(self)
		
		data.fonts['Interface'].Print(500,15, 'Liquid Remaining: ' + str(self.liquidAllowed))
		data.fonts['Interface'].Print(500,30, 'Number of Taps Remaining: ' + str(2 - self.numTapsOn))
		data.fonts['Interface'].Print(500,60, 'Number of Gates Remaining: ' + str(2 - self.numGatesUp))
		
	
	def RetrieveMapInfo(self, level):
		self.myShelf.Open()
		mapinfo = self.myShelf.LoadKey(str(level))
		
		for tap in mapinfo.liquidLocations:
			for tile in map.tiles:
				if tile.x is tap[0] and tile.y is tap[1]:
					self.taps.append(interactables.Tap(tap[0], tap[1], str(tap[2]), tile, self))
			
		for gate in mapinfo.solidLocations:
			for tile in map.tiles:
				if tile.x is gate[0] and tile.y is gate[1]:
					self.gates.append(interactables.Gate(gate[0], gate[1], tile, self))
		
		for finish in mapinfo.finishes:
			for tile in map.tiles:
				if tile.x is finish[0] and tile.y is finish[1]:
					self.finishes.append(interactables.Finish(tile, self))
		
		self.liquidAllowed = mapinfo.liquidAllowed ## do something with this
		
		
		self.myShelf.Close()